﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/01/19 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
// FVXデータ構造の使用宣言
using FVDATASLib;

namespace GrayFilter
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;
        //処理範囲
        private FvRectangle m_ExeRegion = new FvRectangle();


        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //画像を表示
                    axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
                }
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }

        }

        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }

        //カメラ取り込み
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。

            axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

            try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                //画像表示
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImage1.BitPerPixel != 8)
                {
                    MessageBox.Show("グレイ画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImage1.BitPerPixel = 8;
                    axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImage1.ImageWidth = 512;
                    axFvxImage1.ImageHeight = 480;
                }

                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

        //画像ファイル保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }
        }

        //処理範囲設定
        private void btnExeRegion_Click(object sender, EventArgs e)
        {
            int IntOK = 0;
            double dblTemp = 0.0;

            axFvxROI1.SetRectangle(m_ExeRegion);

            // FvxROIを使って処理範囲選択ダイアログを出します｡
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

            if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
            {
                axFvxROI1.GetRectangle(m_ExeRegion);
                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
                // 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
                if (m_ExeRegion.xe < m_ExeRegion.xs)
                {
                    dblTemp = m_ExeRegion.xs;
                    m_ExeRegion.xs = m_ExeRegion.xe;
                    m_ExeRegion.xe = dblTemp;
                }
                if (m_ExeRegion.ye < m_ExeRegion.ys)
                {
                    dblTemp = m_ExeRegion.ys;
                    m_ExeRegion.ys = m_ExeRegion.ye;
                    m_ExeRegion.ye = dblTemp;
                }

                //処理範囲を適用
                axFvxGrayFilter1.SetInspectArea(m_ExeRegion);
                axFvxDisplayEx1.ClearOverlay(-1);
                axFvxDisplayEx1.PenColor = Color.Lime;
                axFvxDisplayEx1.DrawData(0, m_ExeRegion);
            }
        }

        //平均化フィルタ
        private void btnAveraging_Click(object sender, EventArgs e)
        {
            //処理部分が一部の場合に、処理部分以外の元画像が保存されるようコピーします。
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());

            //フィルタ実行
            axFvxGrayFilter1.Averaging(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());

            //平均化処理後に続けて別の処理が出来るように、下のイメージにコピーしています。
            //但し、このコピーを行うと画像の有効範囲がリセットされてしまいます。
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

        }

        //ラプラシアンフィルタ
        private void btnLaplacian_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Laplacian(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //近傍最小値フィルタ
        private void btnMinFilter_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.MinFilter(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //近傍最大値フィルタ
        private void btnMaxFilter_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.MaxFilter(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //4近傍最小値
        private void button5_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Min4Filter(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), FVXCOMGRAYFILTERLib.MINMAX4_MODE_TYPE.ALL_MODE);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //4近傍最大値
        private void btnMax4Filter_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Max4Filter(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), FVXCOMGRAYFILTERLib.MINMAX4_MODE_TYPE.ALL_MODE);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //メディアンフィルタ
        private void btnMedian_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Median(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //鮮鋭化フィルタ
        private void btnSharp_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Sharp(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //微分ソーベルフィルタ
        private void btnSobel_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Sobel(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(),FVXCOMGRAYFILTERLib.DIRECTION_TYPE.XY_DIRECTION);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //微分ロバーツフィルタ
        private void btnRoberts_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Roberts(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //一次微分
        private void btnFdefferential_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Fdefferential(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), FVXCOMGRAYFILTERLib.DIRECTION_TYPE.XY_DIRECTION);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //二次微分
        private void btnSdefferential_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.Sdefferential(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //ゼロクロスフィルタ
        private void btnZeroCross_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.ZeroCross(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 8, 255);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //任意位置クロッシング
        private void btnAnyCross_Click(object sender, EventArgs e)
        {
            axFvxImage1.CopyTo(axFvxImage2.GetOcx());
            axFvxGrayFilter1.AnyCross(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 8, 128, 255);
            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

    }
}